library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity PegasusPicoBlaze is
    Port (
        mclk    : in std_logic;
        btn     : in std_logic_vector(3 downto 0);
        swt     : in std_logic_vector(7 downto 0);
        led     : out std_logic_vector(7 downto 0);
        an      : out std_logic_vector(3 downto 0);
        ssg     : out std_logic_vector(7 downto 0);
        ps2c	 : inout std_logic;
	   ps2d	 : inout std_logic);
end PegasusPicoBlaze ;

architecture Behavioral of PegasusPicoBlaze is

	------------------------------------------------------------------------
	-- Components
	------------------------------------------------------------------------

	component embedded_kcpsm is
	    Port (      
	          port_id : out std_logic_vector(7 downto 0);
          write_strobe : out std_logic;
           read_strobe : out std_logic;
	         out_port : out std_logic_vector(7 downto 0);
               in_port : in std_logic_vector(7 downto 0);
             interrupt : in std_logic;
                 reset : in std_logic;
	              clk : in std_logic);
	end component;

--	component dec_7seg IS
--		PORT(hex_digit						: IN	STD_LOGIC_VECTOR(3 DOWNTO 0);
--			 segment_a, segment_b, segment_c, segment_d, segment_e, 
--			 segment_f, segment_g 			: OUT std_logic);
--		END component;

	component seven_segment_display is
	    Port (    
	         hex_in : in std_logic_vector(3 downto 0);
              segment_a : out std_logic;
              segment_b : out std_logic;
              segment_c : out std_logic;
              segment_d : out std_logic;
              segment_e : out std_logic;
              segment_f : out std_logic;
              segment_g : out std_logic;
              clk : in std_logic );
    end component;
	
    component mouse is
        Port( clock_25Mhz, reset 			: IN std_logic;
          signal mouse_data				: INOUT std_logic;
          signal mouse_clk 				: INOUT std_logic;
        	signal left_button, right_button	: OUT std_logic;
	   	signal mouse_cursor_row 			: OUT std_logic_vector(9 DOWNTO 0); 
	   	signal mouse_cursor_column 		: OUT std_logic_vector(9 DOWNTO 0));       		
    end component;

	------------------------------------------------------------------------
	-- Signals
	------------------------------------------------------------------------

	signal clkdiv  	: std_logic_vector(23 downto 0);
	signal cntr    	: std_logic_vector(3 downto 0);
	signal cclk    	: std_logic;
	signal dig     	: std_logic_vector(7 downto 0);

	signal port_id		: std_logic_vector(7 downto 0);
	signal write_strobe : std_logic;
	signal out_port	: std_logic_vector(7 downto 0);
	signal read_strobe	: std_logic;
	signal in_port		: std_logic_vector(7 downto 0);
	signal interrupt	: std_logic;
	signal reset		: std_logic;

	signal  hostclk : std_logic;	
	signal  mousereset : std_logic;
	signal  mousebtnleft : std_logic;
	signal  mousebtnright : std_logic;
	signal  mouselin : std_logic_vector(9 DOWNTO 0); 
	signal  mousecol : std_logic_vector(9 DOWNTO 0); 
--	signal  restocol : std_logic_vector(1 DOWNTO 0); 
--	signal  restorow : std_logic_vector(1 DOWNTO 0); 

	signal numeric_value: std_logic_vector(3 downto 0);

	

	------------------------------------------------------------------------
	-- Module Implementation
	------------------------------------------------------------------------

	begin

	processor: embedded_kcpsm
	    port map(      
	    	   port_id 		   => port_id,
	  	      write_strobe 	=> write_strobe,
	    	   out_port 		=> out_port,
	         read_strobe 	=> read_strobe,
	         in_port 		   => in_port,
	         interrupt 		=> interrupt,
	         reset 			=> reset,
            clk 			   => cclk);


--	numeric_display: dec_7seg 
--	    port map( 
--	        hex_digit 		=> numeric_value,
--             segment_a 		=> dig(0),
--             segment_b 		=> dig(1),
--             segment_c 		=> dig(2),
--             segment_d 		=> dig(3),
--             segment_e 		=> dig(4),
---            segment_f 		=> dig(5),
--             segment_g		=> dig(6));

	numeric_display: seven_segment_display 
	    port map( 
	          hex_in 		   => numeric_value,
             segment_a 		=> dig(0),
             segment_b 		=> dig(1),
             segment_c 		=> dig(2),
             segment_d 		=> dig(3),
             segment_e 		=> dig(4),
             segment_f 		=> dig(5),
             segment_g		=> dig(6),
             clk 		     => cclk );

	rato: mouse
	    port map( 
		clock_25Mhz 		=> hostclk,
		reset			=> mousereset,
	     mouse_data		=> ps2d,
          mouse_clk 		=> ps2c,
          left_button         => mousebtnleft, 		 
          right_button		=> mousebtnright,
	     mouse_cursor_row	=> mouselin,
	     mouse_cursor_column => mousecol );

	input_ports: process(cclk)
	begin
	  if cclk'event and cclk='1' then
	    case port_id is
	      when "00000000" =>     
	        in_port <= swt;
	      when "11111111" =>    
	        in_port <= ext(btn,8);


	      when "11110000" => ---  F0 BTN LEFT
		   in_port(0)  <= mousebtnleft;
		   in_port(7 downto 1) <= "0000000";
	      when "11110001" => ---  F1 BTN RIGHT
		   in_port(0)  <= mousebtnright;
		   in_port(7 downto 1) <= "0000000";
	      when "11110010" => ---  F2 MOUSE Y
		   in_port(7 downto 0)  <= mousecol(7 downto 0);
--		   restorow		    <= mousecol(1 downto 0);
--		 when "11110011" => ---  F3 MOUSE Y Remaining
--		   in_port(1 downto 0)  <= restorow;

		 when "11110100" => ---  F4 MOUSE X
		   in_port(7 downto 0)  <= mouselin(7 downto 0);
--		   restorow		    <= mouselin(1 downto 0);
--	      when "11110101" => ---  F5 MOUSE X Remaining
--		   in_port(1 downto 0)  <= restorow;
	      when others =>    
	    null;
	    end case;
	  end if; 
	end process input_ports;
	
	output_ports: process(cclk)
	begin
	  if cclk'event and cclk='1' then
	    if write_strobe='1' then
	      case port_id is
	        when "00000000" =>    
		     an  <= "1110";
		     numeric_value <= out_port(3 downto 0);			
	        when "00000001" =>    
		     an  <= "1101";
		     numeric_value <= out_port(3 downto 0);
	        when "00000010" =>    
		     an  <= "1011";
		     numeric_value <= out_port(3 downto 0);
	        when "00000011" =>    
		     an  <= "0111";
		     numeric_value <= out_port(3 downto 0);
	        when "10100000" =>
		     led <= out_port ;
		   when "11110000" => ---F0 reset
		     reset <= out_port(0) ;				   	
	        when others => null;		   
	      end case;
	    else
	      an <= "1111";
	    end if;
	  end if; 
	end process output_ports;
	
	ssg <= not dig;

	process (mclk)
	begin
	    if mclk = '1' and mclk'Event then
	      clkdiv <= clkdiv + 1;
	    end if;
	end process;

	cclk <= clkdiv(2);

	hostclk <= clkdiv(2);

end Behavioral;

