-- Hex to 7-segment display converter
--
-- Version : 1.00
-- Version Date : 29th August 2002
--
-- Ken Chapman
-- Xilinx Ltd
-- Benchmark House
-- 203 Brooklands Road
-- Weybridge
-- Surrey KT13 ORH
-- United Kingdom
--
-- chapman@xilinx.com
--
------------------------------------------------------------------------------------
--
-- NOTICE:
--
-- Copyright Xilinx, Inc. 2002.   This code may be contain portions patented by other 
-- third parites.  By providing this core as one possible implementation of a standard,
-- Xilinx is making no representation that the provided implementation of this standard 
-- is free from any claims of infringement by any third party.  Xilinx expressly 
-- disclaims any warranty with respect to the adequacy of the implementation, including 
-- but not limited to any warranty or representation that the implementation is free 
-- from claims of any third party.  Futhermore, Xilinx is providing this core as a 
-- courtesy to you and suggests that you contact all third parties to obtain the 
-- necessary rights to use this implementation.
--
------------------------------------------------------------------------------------
--
-- Requires 7 LUTs and 7 flip-flops.
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library unisim;
use unisim.vcomponents.all;
--
entity seven_segment_display is
    Port (    hex_in : in std_logic_vector(3 downto 0);
           segment_a : out std_logic;
           segment_b : out std_logic;
           segment_c : out std_logic;
           segment_d : out std_logic;
           segment_e : out std_logic;
           segment_f : out std_logic;
           segment_g : out std_logic;
                 clk : in std_logic );
    end seven_segment_display;
--
architecture low_level_definition of seven_segment_display is
--
-- Internal signals
--
signal decode_seg_a : std_logic;
signal decode_seg_b : std_logic;
signal decode_seg_c : std_logic;
signal decode_seg_d : std_logic;
signal decode_seg_e : std_logic;
signal decode_seg_f : std_logic;
signal decode_seg_g : std_logic;
--
-- Attribute to define LUT contents during implementation 
-- The information is repeated in the generic map for functional simulation
--
attribute INIT : string; 
attribute INIT of decode_a : label is "D7ED";
attribute INIT of decode_b : label is "279F";
attribute INIT of decode_c : label is "2FFB";
attribute INIT of decode_d : label is "7B6D";
attribute INIT of decode_e : label is "FD45";
attribute INIT of decode_f : label is "DF71";
attribute INIT of decode_g : label is "EF7C";
--
begin

     decode_a: LUT4
     --translate_off
     generic map (INIT => X"D7ED")
     --translate_on
     port map( I0 => hex_in(0),
               I1 => hex_in(1),
               I2 => hex_in(2),
               I3 => hex_in(3),
                O => decode_seg_a );

     pipeline_a: FD
     port map ( D => decode_seg_a,
                Q => segment_a,
                C => clk);

     decode_b: LUT4
     --translate_off
     generic map (INIT => X"279F")
     --translate_on
     port map( I0 => hex_in(0),
               I1 => hex_in(1),
               I2 => hex_in(2),
               I3 => hex_in(3),
                O => decode_seg_b );

     pipeline_b: FD
     port map ( D => decode_seg_b,
                Q => segment_b,
                C => clk);

     decode_c: LUT4
     --translate_off
     generic map (INIT => X"2FFB")
     --translate_on
     port map( I0 => hex_in(0),
               I1 => hex_in(1),
               I2 => hex_in(2),
               I3 => hex_in(3),
                O => decode_seg_c );

     pipeline_c: FD
     port map ( D => decode_seg_c,
                Q => segment_c,
                C => clk);

     decode_d: LUT4
     --translate_off
     generic map (INIT => X"7B6D")
     --translate_on
     port map( I0 => hex_in(0),
               I1 => hex_in(1),
               I2 => hex_in(2),
               I3 => hex_in(3),
                O => decode_seg_d );

     pipeline_d: FD
     port map ( D => decode_seg_d,
                Q => segment_d,
                C => clk);

     decode_e: LUT4
     --translate_off
     generic map (INIT => X"FD45")
     --translate_on
     port map( I0 => hex_in(0),
               I1 => hex_in(1),
               I2 => hex_in(2),
               I3 => hex_in(3),
                O => decode_seg_e );

     pipeline_e: FD
     port map ( D => decode_seg_e,
                Q => segment_e,
                C => clk);

     decode_f: LUT4
     --translate_off
     generic map (INIT => X"DF71")
     --translate_on
     port map( I0 => hex_in(0),
               I1 => hex_in(1),
               I2 => hex_in(2),
               I3 => hex_in(3),
                O => decode_seg_f );

     pipeline_f: FD
     port map ( D => decode_seg_f,
                Q => segment_f,
                C => clk);

     decode_g: LUT4
     --translate_off
     generic map (INIT => X"EF7C")
     --translate_on
     port map( I0 => hex_in(0),
               I1 => hex_in(1),
               I2 => hex_in(2),
               I3 => hex_in(3),
                O => decode_seg_g );

     pipeline_g: FD
     port map ( D => decode_seg_g,
                Q => segment_g,
                C => clk);

--
end low_level_definition;
--
------------------------------------------------------------------------------------
--
-- END OF FILE SEVEN_SEGMENT_DISPLAY.VHD
--
------------------------------------------------------------------------------------